<?php
/**
 * The file that defines the core plugin class.
 *
 * @link       https://themehigh.com
 * @since      1.0.0
 *
 * @package    discount-and-dynamic-pricing
 * @subpackage discount-and-dynamic-pricing/includes
 */
if(!defined('WPINC')){	die; }

if(!class_exists('THWDPF')):

class THWDPF {
	protected $plugin_name;
	protected $version;
	const TEXT_DOMAIN = 'discount-and-dynamic-pricing';

	public function __construct() {
		$this->load_dependencies();
		$this->set_plugin_info();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

		add_action('init', array($this, 'init'));
	}

	private function set_plugin_info(){
		$plugin_data = get_plugin_data( THWDPF_FILE );

		$this->plugin_name = 'discount-and-dynamic-pricing';
		$this->version = isset($plugin_data['Version']) ? $plugin_data['Version'] : '1.0.0';
		define('THWDPF_VERSION', $this->version);
	}

	private function load_dependencies() {
		if(!function_exists('is_plugin_active')){
			include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
		}

		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-thwdpf-autoloader.php';
	}

	private function set_locale() {
		add_action('plugins_loaded', array($this, 'load_plugin_textdomain'));
	}

	public function load_plugin_textdomain(){
		$locale = apply_filters('plugin_locale', get_locale(), self::TEXT_DOMAIN);
	
		load_textdomain(self::TEXT_DOMAIN, WP_LANG_DIR.'/discount-and-dynamic-pricing/'.self::TEXT_DOMAIN.'-'.$locale.'.mo');
		load_plugin_textdomain(self::TEXT_DOMAIN, false, dirname(THWDPF_BASE_NAME) . '/languages/');
	}

	private function define_admin_hooks() {
		$plugin_admin = new THWDPF_Admin( $this->get_plugin_name(), $this->get_version() );

		add_action('admin_enqueue_scripts', array($plugin_admin, 'enqueue_styles_and_scripts'));
		add_action('admin_menu', array($plugin_admin, 'admin_menu'));
		add_filter('woocommerce_screen_ids', array($plugin_admin, 'add_screen_id'));
		add_filter('plugin_action_links_'.THWDPF_BASE_NAME, array($plugin_admin, 'plugin_action_links'));
		//add_filter('plugin_row_meta', array($plugin_admin, 'plugin_row_meta'), 10, 2);

		$wdpf_data = THWDPF_Data::instance();
	}

	private function define_public_hooks() {
		$discount_product = new THWDPF_Public_Discount_Product($this->get_plugin_name(), $this->get_version());
		$discount_cart = new THWDPF_Public_Discount_Cart($this->get_plugin_name(), $this->get_version());
	}

	public function init(){
		$this->define_constants();
	}
	
	private function define_constants(){
		!defined('THWDPF_ASSETS_URL_ADMIN') && define('THWDPF_ASSETS_URL_ADMIN', THWDPF_URL . 'admin/assets/');
		!defined('THWDPF_ASSETS_URL_PUBLIC') && define('THWDPF_ASSETS_URL_PUBLIC', THWDPF_URL . 'public/assets/');
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}
}

endif;