<?php
/**
 * Class to handles product specific discount calculations for frontend.
 *
 * @link       https://themehigh.com
 *
 * @package    discount-and-dynamic-pricing
 * @subpackage discount-and-dynamic-pricing/public
 */

if(!defined('WPINC')){ die; }

if(!class_exists('THWDPF_Public_Discount_Product')):

class THWDPF_Public_Discount_Product extends THWDPF_Public_Discount{
	private $price;

	public function __construct($plugin_name, $version) {
		parent::__construct($plugin_name, $version, 'product');
		add_action('after_setup_theme', array($this, 'define_public_hooks'));
	}

	public function define_public_hooks(){
		//add_action('woocommerce_before_add_to_cart_button', array($this, 'render_bulk_discount_info'));
		//add_action('woocommerce_before_add_to_cart_button', array($this, 'render_price_table'));

		add_filter('woocommerce_add_cart_item_data', array($this, 'woo_add_cart_item_data'), 10, 4);
		add_action('woocommerce_before_calculate_totals', array($this, 'apply_product_discounts'), 10, 1);
		add_filter('woocommerce_cart_item_price', array($this, 'woo_cart_item_price'), 10, 3);

		// add_action('woocommerce_add_to_cart', array($this, 'set_cart_totals_bkp'), 20);
		// add_action('woocommerce_cart_item_removed', array($this, 'set_cart_totals_bkp'));
		// add_action('woocommerce_cart_item_restored', array($this, 'set_cart_totals_bkp'));
		// add_action('woocommerce_after_cart_item_quantity_update', array($this, 'set_cart_totals_bkp'));
		// //add_action('woocommerce_after_calculate_totals', array($this, 'remove_cart_totals_bkp'), 10, 1);
	}

	public function woo_add_cart_item_data($cart_item_data, $product_id=0, $variation_id=0, $quantity=0){
		$product = wc_get_product( $variation_id ? $variation_id : $product_id );
		$discount_data = $this->get_product_discount_data($product, $quantity);

		if($discount_data){
			$cart_item_data[self::ITEM_KEY_DISCOUNTS] = $discount_data;
		}

		return $cart_item_data;
	}

	public function apply_product_discounts($cart_object){
		foreach($cart_object->cart_contents as $key => &$item) {
			$data = isset($item[self::ITEM_KEY_DISCOUNTS]) ? $item[self::ITEM_KEY_DISCOUNTS] : false;

			if(is_array($data) && THWDPF_Utils::is_valid_cart_item($item)){
			$discount_rules = isset($data['discount_rules']) ? $data['discount_rules'] : false;
			$original_price = isset($data['original_price']) ? $data['original_price'] : false;

			$discount_data = $this->prepare_discount_data($discount_rules, $item, $original_price);
			$final_price = isset($discount_data['final_price']) ? $discount_data['final_price'] : false;

			if(is_numeric($final_price)){
				$item['data']->set_price($final_price);
				$item['data']->set_sale_price($final_price);
			}
			}
		}
	}

	public function woo_cart_item_price($price, $cart_item, $cart_item_key){
		$product = isset($cart_item['data']) ? $cart_item['data'] : false;

		if($product && $product->is_on_sale()) {
			$reg_price = $this->get_product_price( $product, array('price' => $product->get_regular_price()) );
			$sale_price = $this->get_product_price( $product );

			$price = wc_format_sale_price($reg_price, $sale_price) . $product->get_price_suffix();
			$price = apply_filters( 'thwdpf_cart_product_price_html', $price, $product );
		}

		return $price;
	}

	private function get_product_price($product, $args = array()) {
		if ( WC()->cart->display_prices_including_tax() ) {
			$product_price = wc_get_price_including_tax( $product, $args );
		} else {
			$product_price = wc_get_price_excluding_tax( $product, $args );
		}
		return apply_filters( 'thwdpf_cart_product_price', wc_price( $product_price ), $product );
	}

	/*******************************************
	 ***** PREPARE DISCOUNT DATA - START *******
	 *******************************************/
	private function get_valid_discount_rules($product, $quantity=false){
		$valid_rules = array();
		$rules = THWDPF_Utils::get_product_rules();
		//$rules = $this->get_qualified_rules($rules, $prod_summary);

		if(is_array($rules)){
			$prod_summary = THWDPF_Utils::get_product_summary($product, $quantity);
			$is_logged_in = is_user_logged_in();
			$user  = wp_get_current_user();
			$roles = THWDPF_Utils::get_user_roles($user);
			$today = current_datetime();
		
			foreach ($rules as $name => $rule) {
				if(THWDPF_Utils::is_valid_enabled($rule)){
					if(!$this->is_active($rule, $today)){
						continue;
					}

					if(!$this->is_authorized($rule, $is_logged_in, $user, $roles)){
						continue;
					}

					$restrictions = $rule->get_property('buy_restrictions');

					if(!$this->is_qualified_product($restrictions, $prod_summary)){
						continue;
					}

					if(!$this->is_qualified_category($restrictions, $prod_summary)){
						continue;
					}

					$valid_rules[$name] = $rule;
				}
			}
			//$valid_rules = $this->filter_by_apply_when($valid_rules);
		}

		return $valid_rules;
	}

	private function filter_discount_rules($rules, $product, $quantity=0){
		$valid_rules = array();

		if(is_array($rules)){
			$args = THWDPF_Utils::get_product_summary($product, $quantity);
		
			foreach ($rules as $name => $rule) {
				if(THWDPF_Utils::is_valid_enabled($rule)){
					$restrictions = $rule->get_property('buy_restrictions');

					if(!$this->is_qualified_others($restrictions, $args)){
						continue;
					}

					$valid_rules[$name] = $rule;
				}
			}

			$valid_rules = $this->filter_by_apply_when($valid_rules);
		}
		
		return $valid_rules;
	}
	
	private function get_product_discount_data($product, $quantity=1){
		$rules = $this->get_valid_discount_rules($product, $quantity);

		$discount_data = array(
			'discount_rules' => $rules,
			'original_price' => $product->get_price(),
		);

		return $discount_data;
	}

	private function prepare_discount_data($discount_rules, $item, $prod_price){
		$discount_data = array();
		
		if(is_array($discount_rules)){
			$product = isset($item['data']) ? $item['data'] : false;
			$prod_qty = isset($item['quantity']) ? $item['quantity'] : false;

			if(is_numeric($prod_price) && is_numeric($prod_qty) && $prod_qty > 0){
				$total_discount = 0;
				$discount_rules = $this->filter_discount_rules($discount_rules, $product, $prod_qty);

				foreach ($discount_rules as $name => $rule) {
					$discount = $this->calculate_product_discount($rule, $prod_price, $prod_qty);
					
					if($discount > 0){
						$total_discount += $discount;
					}
				}

				$final_price = $total_discount > 0 ? $prod_price-$total_discount : $prod_price;

				$discount_data['product_qty'] = $prod_qty;
				$discount_data['product_price'] = $prod_price;
				$discount_data['total_discount'] = $total_discount;
				$discount_data['final_price'] = $final_price;
			}
		}

		return empty($discount_data) ? false : $discount_data;
	}

	private function calculate_product_discount($rule, $prod_price, $prod_qty){
		$discount = 0;
		
		if(THWDPF_Utils::is_valid_enabled($rule)){
			$method = $rule->get_property('method');

			if($method === 'simple'){
				$disc_type = $rule->get_property('discount_type');
    			$disc_amount = $rule->get_property('discount_amount');

    			$discount = $this->calculate_discount($disc_type, $disc_amount, $prod_price);

			}else if($method === 'bulk'){
				$range_discounts = $rule->get_property('range_discounts');

				if(is_array($range_discounts)){
					foreach ($range_discounts as $key => $range) {
						$is_valid_range = false;
						
						$min_qty = isset($range['min_qty']) && is_numeric($range['min_qty']) ? $range['min_qty'] : 0;
						$max_qty = isset($range['max_qty']) && is_numeric($range['max_qty']) ? $range['max_qty'] : 0;

						if($max_qty > 0 && $min_qty >= 0){
							if($max_qty > $min_qty){
								if($prod_qty >= $min_qty && $prod_qty <= $max_qty){
									$is_valid_range = true;
								}
							}
						}else if($max_qty > 0){
							if($prod_qty <= $max_qty){
								$is_valid_range = true;
							}
						}else if($min_qty >= 0){
							if($prod_qty >= $min_qty){
								$is_valid_range = true;
							}
						}

						if($is_valid_range){
							$disc_type = isset($range['discount_type']) ? $range['discount_type'] :'';
    						$disc_amount = isset($range['discount']) ? $range['discount'] : 0;

							$discount = $this->calculate_discount($disc_type, $disc_amount, $prod_price);
							break;
						}
					}
				}
			}
		}

		return $discount;
	}
	
	/*public function set_cart_totals_bkp(){
		THWDPF_Utils::set_cart_totals_bkp(WC()->cart);
	}*/

	/*public function remove_cart_totals_bkp($cart){
		//THWDPF_Utils::set_cart_totals_bkp(WC()->cart, 'add');
		//THWDPF_Utils::remove_cart_totals_bkp($cart);
	}*/
	/*******************************************
	 ***** PREPARE DISCOUNT DATA - END *********
	 *******************************************/


	/*****************************************
	 ********* PRICE DISPLAY - START *********
	 *****************************************/
	/*private function is_show_price_table($settings=false){
		$show = THWDPF_Utils::get_setting_value($settings, 'show_price_table');
		$show = apply_filters('thwdpf_show_price_table', $show);
		$show = true; //TODO remove this line

		return $show;
	}

	public function render_bulk_discount_info(){		
		global $product;
		$rules = $this->get_valid_discount_rules($product, 1);

		foreach ($rules as $name => $rule) {
			if(THWDPF_Utils::is_valid_enabled($rule)){
				$method = $rule->get_property('method');

				if($method === 'bulk'){
					$range_discounts = $rule->get_property('range_discounts');

					if(is_array($range_discounts) && !empty($range_discounts)){
						$cell_html_qty = '';
						$cell_html_amt = '';

						foreach ($range_discounts as $key => $range) {
							$min_qty = isset($range['min_qty']) ? $range['min_qty'] : 0;
							$max_qty = isset($range['max_qty']) ? $range['max_qty'] : 0;

							if($max_qty > $min_qty){
								$dtype = isset($range['discount_type']) ? $range['discount_type'] :'';
		    					$damount = isset($range['discount']) ? $range['discount'] : 0;
		    					$damount = $dtype === '' ? $damount.'%' : wc_price($damount);

								$cell_html_qty .= '<td class="qty">'.$min_qty.'-'.$max_qty.'</td>';
								$cell_html_amt .= '<td class="amt">'.$damount.'</td>';
							}
						}

						if($cell_html_qty && $cell_html_amt){
							?>
							<table class="thwdpf-bulk-discount-info"><tbody>
								<tr class="row-qty">
									<td class="label"><?php esc_html_e('Quantity', 'discount-and-dynamic-pricing'); ?></td>
									<?php echo $cell_html_qty; ?>
								</tr>
								<tr class="row-amt">
									<td class="label"><?php esc_html_e('Price', 'discount-and-dynamic-pricing'); ?></td>
									<?php echo $cell_html_amt; ?>
								</tr>
							</tbody></table>
							<?php
						}
					}
				}
			}
		}
	}*/

	//Display Price Table
	/*public function render_price_table(){
		$settings = THWDPF_Utils::get_advanced_settings();

		if($this->is_show_price_table($settings)){
			global $product;
			$discount_data = $this->get_product_discount_data($product, 1);
			
			if(is_array($discount_data)){
				$price = $product->get_price_html();
				$product_price  = isset($discount_data['product_price']) ? $discount_data['product_price'] : false;
				$discount_final = isset($discount_data['discount_final']) ? $discount_data['discount_final'] : false;
				$final_price    = isset($discount_data['final_price']) ? $discount_data['final_price'] : false;

				$label_discount = THWDPF_Utils::get_setting_value($settings, 'pt_discount_label', 'Discount');
				$label_price    = THWDPF_Utils::get_setting_value($settings, 'pt_price_label', 'Product Price');
				$label_total    = THWDPF_Utils::get_setting_value($settings, 'pt_total_label', 'Total');
				$is_div_model   = THWDPF_Utils::get_setting_value($settings, 'use_div_model');

				if($discount_final > 0){
					if($is_div_model === 'yes'){
						?>
						<h2><?php esc_html_e('Price Table', 'discount-and-dynamic-pricing'); ?></h2>
						<div class="thwdpf-price-table" style="display: block">
							<div class="table-item extra-price">
								<div class="label"><?php esc_html_e($label_discount, 'discount-and-dynamic-pricing'); ?></div>
								<div class="value"><?php echo esc_html($discount_final); ?></div>
							</div>
							<div class="table-item product-price">
								<div class="label"><?php esc_html_e($label_price, 'discount-and-dynamic-pricing'); ?></div>
								<div class="value"><?php echo esc_html($price); ?></div>
							</div>
							<div class="table-item total-price">
								<div class="label"><?php esc_html_e($label_total, 'discount-and-dynamic-pricing'); ?></div>
								<div class="value" data-price="<?php echo esc_html($final_price); ?>"><?php echo esc_html($final_price); ?></div>
							</div>
						</div>
						<?php
					}else{
						?>
						<h2><?php esc_html_e('Price Table', 'discount-and-dynamic-pricing'); ?></h2>
						<table class="thwdpf-price-table" style="display: block">
							<tbody>
								<tr class="extra-price">
									<td class="label"><?php esc_html_e($label_discount, 'discount-and-dynamic-pricing'); ?></td>
									<td class="value"><?php echo esc_html($discount_final); ?></td>
								</tr>
								<tr class="product-price">
									<td class="label"><?php esc_html_e($label_price, 'discount-and-dynamic-pricing'); ?></td>
									<td class="value"><?php echo esc_html($price); ?></td>
								</tr>
								<tr class="total-price">
									<td class="label"><?php esc_html_e($label_total, 'discount-and-dynamic-pricing'); ?></td>
									<td class="value" data-price="<?php echo esc_html($final_price); ?>"><?php echo esc_html($final_price); ?></td>
								</tr>
							</tbody>
						</table>
						<?php
					}
				}
			}
		}
	}*/
	/****************************************
	 ********* PRICE DISPLAY - END **********
	 ****************************************/
}

endif;